Unit BBSDEF;
(*

  This unit is (C) 1989, Gallagher & Robertson, A/S, and may not be
  distributed in any modified form whatsoever without our permission.
  Any program making use of these definitions may be distributed freely
  but *ONLY* if the complete source of such programs has been checked
  by Gallagher & Robertson for correctness. In addition, no charge may
  be made for distribution of such programs without our express
  permission.
  
  These rules are for the protection of our users, in order that we may
  ensure that programs which access BBS files do so in an 'orderly
  fashion'. Any infringement of these rules will be treated as an
  illegal action and will be prosecuted by all means at our disposal.

  *)



{ BBSDEF.PAS -------------------------------------------------------------------
    This module contains all 'global' constant and type definitions for the
    BBS system.
------------------------------------------------------------------------------ }

interface

{ general usage TYPE definitions --------------------------------------------- }

type
  NameT      = String[30];                           {      - " -              }
  PassT      = String[8];                            { for PASSWORD            }
  MenuNameT  = String[12];                           { name of a menu          }
  TelNoT     = String[16];                           { for Telephone numbers   }
  ModemStrT  = String[60];                           { for modem strings       }
  PathNameT  = String[60];                           { path names              }

{ Compile-time constants ----------------------------------------------------- }

const
  MaxConferences  = 75;         { absolute maximum, must recompile to get more }
  MaxFileDirs     = 99;         { maximum number of file directories           }

{ general usage record TYPE definitions -------------------------------------- }

type
  DayOfYearT = record                                { a DATE                  }
    year,
    month,
    day     : byte;
  end {DayOfYearT};

  TimeOfDayT = record                                { a TIME                  }
    hour,
    minute,
    second  : byte;
  end {TimeOfDayT};

  DateTimeT = record                                 { DATE and TIME           }
    date    : DayOfYearT;
    time    : TimeOfDayT;
  end {DateTimeT};

{ miscellaneous definitions ---------------------------------------------------}

type
  AccessTypes  = (ReadMessage, WriteMessage, SendFiles, ReceiveFiles,
                   SigopFunctions, SysopFunctions, OpenDoors, RFU);
  AccessType   = set of AccessTypes;
  ProtocolT    = (NoProtocol,
                  AsciiP, XModemP, XModemCRCP, YModemP, ZModemP, ZModemP1,
                  ZModemP2, KermitP, YmodemBP, YmodemGP, Modem7P, TelinkP);
  MenuT        = (NoviceM, ExpertM, JuniorM, SuperM);
  TerminalT    = (TTY, ANS, VIP);
  MsgReadT     = (marked, forwards, backwards, reference);
  ConfSwitchT  = (ImmRead,ImmWrite,Mail,Pmess,UseAlias,ImmJoin,Obljoin,Resume);
  ConfSwitches = set of ConfSwitchT;
  CharSets     = (IBNchars, US7chars, ISOchars, IBMchars, 
                  UK7chars, GE7chars, SF7chars, NO7chars,
                  DE7chars, IT7chars, FR7chars, SP7chars);

{ definitions for the USERS file --------------------------------------------- }

type
  ConfT        = record                              { for user record         }
    LastRead   : integer;                            { value -2 = invited      }
                                                     { value -1 = nonmember    }
    Access     : AccessType;                         { security this conference}
  end {ConfType};

  UserRecord   = record
    Name       : NameT;                              { First AND Last name     }
    Address    : NameT;                              { Street address          }
    CityState  : NameT;                              { City/State              }
    Password   : PassT;                              { users password          }
    LastAccess : DateTimeT;                          { last time on system     }
    FilesDate  : DateTimeT;                          { last date files checked }
    TimesOn    : integer;                            { number of times on      }
    Uploaded   : integer;                            { files uploaded          }
    Downloaded : integer;                            { files downloaded        }
    PageLength : integer;                            { page length             }
    TimeUsed   : integer;                            { minutes on system today }
    Protocol   : ProtocolT;                          { download protocol       }
    HomeTelno  : TelnoT;                             { home telephone          }
    WorkTelno  : TelnoT;                             { work telephone          }
    Menutype   : MenuT;                              { menu display type       }
    ANSI       : boolean;                            { wants ANSI graphics     }
    FullScreen : boolean;                            { wants full screen edit  }
    TermType   : TerminalT;                          { terminal type           }
    Charset    : Charsets;                           { character set           }
    OvrMode    : boolean;                            { overlay in editor       }
    NoRowCol   : boolean;                            { turned off rowcol       }
    TimeLimit  : integer;                            { daily time limit        }
    NoShow     : boolean;                            { no conference show      }
    Killed     : boolean;                            { user is dead            }
    NoMsgCol   : boolean;                            { no colours in msgs      }
    ReadType   : MsgReadT;                           { dflt read type          }
    AutoXFR    : byte;                               { more coming?            }
    Polls      : integer;                            { one bit per poll        }
    LangKey    : char;                               { language key if any     }
    Filtering  : boolean;                            { filtering noise         }
    Resume     : integer;                            { resume number           }
    ResumeAdr  : boolean;                            { add address to resume   }
    ATimesOn   : integer;                            { times on last reset     }
    AUploaded  : integer;                            { uploaded last reset     }
    ADownloaded: integer;                            { downloaded last reset   }
    AMsgsLeft  : integer;                            { messages entered lst res}
    MsgsLeft   : integer;                            { messages entered        }
    MsgsRead   : integer;                            { messages read           }
    MsgsDumped : integer;                            { messages dumped         }
    LastBumped : DateTimeT;                          { last thrown out         }
    Alias      : NameT;
    ShortForm  : boolean;                            { short message format    }
    FileUsed   : integer;                            { minutes in file menu    }
    FileLimit  : integer;                            { daily file limit        }
    OldIndent  : boolean;                            { autoindent NOT USED     }
    MainScript : String[12];                         { script to use if any    }
    Encoded    : boolean;                            { use encoded Xmodems     }
    MsgFilter  : byte;                               { system msg filter       }
    Totaltime  : longint;                            { total time on system    }
    Atotaltime : longint;                            { at last reset           }
    BytesDown  : longint;                            { downloaded bytes        }
    AbytesDown : longint;                            { at last reset           }
    SendPref   : (SendARC, SendTXT, SendZIP);        { preference for SEND     }
    Spare      : array [1..15] of char;              { RFU                     }
    Conf       : array [0..MaxConferences] of ConfT; { zero is for MAIN        }
  end {UserRecord};

{ definitions for the FILES file --------------------------------------------- }

  FileRecord   = record
    Name       : MenuNameT;                          { Short name of file      }
    PathName   : PathNameT;                          { Where to find file      }
    Description: String[40];                         { File description        }
    DateStamp  : DateTimeT;                          { date/time of file       }
    FileSiz    : longint;                            { size of file            }
    FileFrom   : integer;                            { submitter               }
    FileTo     : integer;                            { receiver, if DirNum <0  }
    Downloads  : longint;                            { times downloaded        }
    ASCII      : boolean;                            { may be ASCII d'ld       }
    DirNum     : integer;                            { directory number        }
                                                     { -1 for privates         }
    ConfNo     : integer;                            { private to conference   }
    Available  : boolean;                            { private altogether      }
    FileDesc   : integer;                            { ptr to message w/desc   }
    Notify     : boolean;                            { notify uploader         }
    Reserve    : array [1..114] of byte;             { future use              }
  end {FileRecord};

{ definitions for the MESSAGES file ------------------------------------------ }

type
  MsgStatusT   = (NormalMsg, KilledByAuthor, KilledBySysop, ArchivedMsg,
                  MovedBySysop);
  SecureT      = (SecNone, SecPassword, SecReceiver);
  MessageLine  = String[72];

  MessageRecord   = record
    Number     : integer;                            { Message Number          }
    MsgStatus  : MsgStatusT;                         { Message Status          }
    Security   : SecureT;                            { Message security        }
    MsgFrom    : integer;                            { user number             }
    MsgTo      : integer;                            { Ditto (-1=ALL)          }
    Subject    : NameT;                              { subject of message      }
    TimeStamp  : DateTimeT;                          { time entered            }
    MsgRead    : boolean;                            { read by receiver        }
    RefTo      : integer;                            { refers to               }
    RefBy      : integer;                            { first answer            }
    RefNxt     : integer;                            { next in this thread     }
    NrLines    : integer;                            { number of lines         }
    NrBytes    : word;                               { number of bytes         }
    TextOffs   : real;                               { offset in text file     }
  end {MessageRecord};

{ The configuration record definition ---------------------------------------- }

type
  EventT = record
    EventName          : NameT;
    WeekDay            : word;
    Earliest           : TimeOfDayT;
    Latest             : TimeofDayT;
    LastRun            : DateTimeT;
    BusyPhone          : boolean;
  end {EventT};

type
  ConfigRecord = record
    CFGID              : array [1..4] of char;
    CFGlevel           : word;
    BaseName           : NameT;       { name of this BBS }
    SYSOPname          : NameT;
    SYSOPpassword      : PassT;
    CommsPort          : word;
    OpenModemAt        : word;
    ModemInitString    : ModemStrT;
    ModemAnswerString  : ModemStrT;
    ModemOffHookString : ModemStrT;
    ModemOnHookString  : ModemStrT;
    MaxLinesMessage    : word;
    MaxActiveConf      : word;     { limited to MaxConferences }
    MaxActiveDirs      : word;     { limited to MaxFileDirs }
    NewUserTimeLimit   : word;
    UploadsPrivate     : boolean;
    SleepTime          : word;
    NoLongerUsed       : boolean;
    NumberOfEvents     : word;
    MainDIR            : PathNameT;
    MenuDIR1           : PathNameT;
    FileDIR            : PathNameT;
    AllowARC           : boolean;
    AllowZmodem        : boolean;
    MenuDIR2           : PathNameT;
    ScratchPath        : PathNameT;
    CallersPath        : PathNameT;
    SysopsPath         : PathNameT;
    NodesPath          : PathNameT;
    RemotePath         : PathNameT;    
    NodeNumber         : word;
    MaxNodes           : word;
    ImmUpload          : boolean;
    ImmDownload        : boolean;
    ClosedSystem       : boolean;
    ClosedPassword     : PassT;
    DirectSystem       : boolean;    
    SpaceRequired      : word;
    ArcCommand         : PathNameT;
    DszCommand         : PathNameT;
    AllowKermit        : boolean;
    KermitCommand      : PathNameT;
    UseAlias           : boolean;
    HoldDIR            : PathNameT;
    ArxCommand         : PathNameT;
    NorwegianChars     : boolean;
    DefaultCharSet     : CharSets;
    NoOutputInt        : boolean;
    DefaultByRef       : boolean;
    ConstantSpeed      : boolean;
    CommsPortBase      : word;
    CommsPortIRQ       : word;
    ConfNames          : array [0..MaxConferences] of NameT;
    ConfSW             : array [0..MaxConferences] of ConfSwitches;
    ConfOrder          : array [0..MaxConferences] of byte;
    ConfDefaultMsg     : array [0..MaxConferences] of word;
    DirNames           : array [0..MaxFileDirs] of NameT;
    SysopMacros        : array [1..10] of NameT;
    Events             : array [1..20] of EventT;
    TimePeriods        : array [0..23] of word;
    Quarantines        : array [0..23] of word;
    FileOrder          : array [0..MaxFileDirs] of byte;
    DirectConnect      : boolean;
    NoCarrierPause     : word;
    NumberDoors        : word;
    ChatDIR            : PathNameT;
    PollTime           : word;
    NewUserFileLimit   : word;
    ImmDownConf        : word;
    DefaultScript      : String[12];
    ModemCallString    : ModemStrT;
    NoFkeys            : boolean;
    LocalSysop         : boolean;
    SwapDIR            : PathNameT;
    Dummy              : boolean;
    NewUserOpen        : boolean;
    AllowZIP           : boolean;
    ZIPcommand         : PathNameT;
    UNZIPcommand       : PathNameT; 
    SuppressLocal      : boolean;
    InitiallyAvailable : boolean;
    DontEcho           : boolean;
    CarrierDetect      : boolean;
    RestrictByConf     : word;
    ModemNoDTR         : boolean;
    ModemDropString    : ModemStrT;
  end {ConfigRecord};

(* Generate a user key for the index file from a given name *)

function UserKeyGen (N : NameT) : word; 

implementation

{ UserKeyGen -------------------------------------------------------------------
    Generate the key for a given name.
  ---------------------------------------------------------------------------- }
function UserKeyGen (N : NameT) : word;

  var
    Offset  : integer;
    UserKey : word;

  function MappedLetter : integer;
    var
      C : char;
    begin
      if Offset > length(N) then
        MappedLetter := 0
      else
        begin
          C := N[Offset];
          case C of
            'A'..'Z' : MappedLetter := ord(C)-$40;
            '',
            '',
            ''      : MappedLetter := 1;
            ''      : MappedLetter := 3;
            ''      : MappedLetter := 5;
            '',
            ''      : MappedLetter := 15;
            ''      : MappedLetter := 21;
            else       MappedLetter := 31;
          end;
        end;
    end {MappedLetter};

  begin
    Offset := 1;
    UserKey := MappedLetter shl 10;
    repeat
      inc(Offset);
    until (Offset>length(N)) or (N[Offset]=' ');
    repeat
      inc(Offset);
    until (Offset>length(N)) or (N[Offset]<>' ');
    UserKey := UserKey or (MappedLetter shl 5);
    inc(Offset);
    UserKey := UserKey or MappedLetter;
    UserKeyGen := UserKey;
  end {UserKeyGen}; 

begin
end {BBSDEF}.


